#include <hmg.ch>

Function Main
   local aPorts := {}
   local i
   private s_nHandle 
   private s_lConnected := .f.
   private lModemFound := .f.

   for i := 1 to 25
      aadd( aPorts, 'COM' + alltrim( str( i ) ) )
   next i
   
   define window sms at 0, 0 width 800 height 350 title 'HMG SMS' main
      define frame modemframe
         row 10
         col 5
         width 790
         height 105
         caption 'Modem Configuration'
      end frame
      define label modemlabel
         row 30
         col 10
         width 200
         value 'Modem connected to port'
      end label
      define combobox comports
         row 30
         col 210
         width 70
         items aPorts
         onchange AutoDetectPortChanged()
         value 1
      end combobox
      define button autodetectbut
         row 30
         col 290
         caption 'Detect Modem'
         width 100
         action detectmodem()
      end button
      define checkbox autodetect
         row 30
         col 400
         caption 'Auto Detect'
         width 100
         onchange AutoDetectPortChanged()
      end checkbox
      define label modemstatus
         row 30
         col 520
         width 270
         value 'Modem Not Verified'
         fontcolor { 255, 0, 0 }
         fontbold .t.
         fontsize 12
      end label
      define label trieslabel
         row 60
         col 10
         width 200
         value 'Number of Tries'
      end label
      define spinner tries
         row 60
         col 210
         width 50
         rangemin 1
         rangemax 10
         value 3
      end spinner
      define label waitlabel
         row 60
         col 290
         width 100
         value 'Waiting time'
      end label
      define spinner wait
         row 60
         col 400
         width 50
         rangemin 1
         rangemax 10
         value 2
      end spinner
      define label seconds
         row 60
         col 460
         width 50
         value 'Seconds'
      end label         
      define label statuslabel
         row 90
         col 10
         width 200
         value 'Status'
      end label
      define label status
         row 90
         col 210
         width 380
         fontcolor { 0, 0, 255 }
         fontbold .t.
      end label
      define label phonenumberlabel
         row 120
         col 10
         width 200
         value "Receipient's Mobile No."
      end label
      define textbox phoneno
         row 120
         col 210
         width 200
      end textbox
      define label messagelabel
         row 150
         col 10
         width 200
         value 'Enter your message here' 
      end label
      define editbox message
         row 150
         col 210
         width 380
         height 100
      end editbox
      define button send
         row 270
         col 310
         width 80
         caption 'Send SMS'
         action send_sms()
      end button
      define button close
         row 270
         col 410
         width 80
         caption 'Close'
         action sms.release()
      end button
   end window 
   on key ESCAPE of sms action sms.release()
   sms.center()
   sms.activate()   
Return nil

function send_sms
   local cPort
   local nLineNo := sms.comports.value
   local cMessage := sms.message.value
   local cPhoneNo := sms.phoneno.value
   if .not. lModemFound
      msgstop( 'Modem is not verified. Either select the port and click <Detect Modem> or check <auto detect>' )
      return nil
   endif
   if nLineNo > 0
      cPort := sms.comports.item( nLineNo )
   endif
   // step 1 : Try Connecting to modem
   sms.status.value := 'Connecting Modem in ' + cPort
   s_nHandle := INIT_PORT( cPort, 9600, 8, 0, 1, 64 )   
   if s_nHandle > 0
      if isworking( s_nHandle )
         if _sms_send_at( "ATE0V1Q0" + Chr( 13 ) ) // try to find modem
            sms.status.value := 'Connected Modem in ' + cPort
         else
            sms.status.value := 'Modem initialization failed.'
            _sms_conn_close()   
            return nil
         endif
      endif
      // step 2 : set modem to send sms in text mode
      sms.status.value := 'Initializing Message mode...'
      if isworking( s_nHandle )
         if _sms_send_at( "AT+CMGF=1" + Chr( 13 ) )
            sms.status.value := 'Message mode initialized successfully'
         else   
            sms.status.value := 'Message mode initialization failed'
            _sms_conn_close()   
            return nil
         endif   
      endif
      // step 3 : 
      sms.status.value := 'Sending phone number to the modem'
      if isworking( s_nHandle )
         if _sms_init_msg( 'AT+CMGS="' + cPhoneNo + '"' + Chr( 13 ) )
            sms.status.value := 'Modem is ready to send the message'
         else
            sms.status.value := 'Modem response not received'
            _sms_conn_close()   
            return nil
         endif   
      endif
      // step 4
      sms.status.value := 'Sending message...'
      if isworking( s_nHandle )
         if _sms_send_msg( StrTran( cMessage, Chr( 13 ) ) + Chr( 26 ) )
            sms.status.value := 'Message Sent successfully'
         else
            sms.status.value := 'Message sent but verification not received.'
            _sms_conn_close()   
            return nil
         endif
      endif
      if isworking( s_nHandle )
         if _sms_send_at( "AT" + Chr( 13 ) )
            _sms_conn_close()   
            sms.status.value := 'Modem connection is reset and ready again'
         endif   
      endif
      _sms_conn_close()
   endif
   return nil   
        
   



return nil

function AutoDetectPortChanged
   local cPort, i 
   if sms.autodetect.value
      lModemFound := .f.
      sms.comports.enabled := .f.
      sms.modemstatus.value := 'Modem Not Verified'
      sms.modemstatus.fontcolor := { 255, 0, 0 }
      sms.status.value := 'Auto detecting the modem'
      for i := 1 to 25
         cPort := 'COM' + alltrim( str( i ) )
         sms.status.value := 'Trying to find in ' + cPort
         s_nHandle := INIT_PORT( cPort, 9600, 8, 0, 1, 64 )   
         if s_nHandle > 0
            if isworking( s_nHandle )
               if _sms_send_at( "ATE0V1Q0" + Chr( 13 ) ) // try to find modem
                  sms.modemstatus.value := 'Modem found in ' + cPort
                  sms.comports.value := i
                  sms.modemstatus.fontcolor := { 50, 100, 0 }
                  lModemFound := .t.
                  exit
               endif
            endif
            _sms_conn_close()
         endif
      next i
      if .not. lModemFound
         sms.status.value := 'Modem could not be found'
      else
         sms.status.value := ''      
      endif   
   else
      sms.comports.enabled := .t.
      lModemFound := .f.
      sms.modemstatus.value := 'Modem Not Verified'
      sms.modemstatus.fontcolor := { 255, 0, 0 }
      sms.status.value := 'Please click <Detect Modem> to verify'
   endif
return nil

function detectmodem
   local cPort := ''
   local nLineNo := sms.comports.value
   if nLineNo > 0
      lModemFound := .f.
      cPort := sms.comports.item( nLineNo )
      sms.status.value := 'Trying to find in ' + cPort
      s_nHandle := INIT_PORT( cPort, 9600, 8, 0, 1, 64 )   
      if s_nHandle > 0
         if isworking( s_nHandle )
            if _sms_send_at( "ATE0V1Q0" + Chr( 13 ) ) // try to find modem
               sms.modemstatus.value := 'Modem found in ' + cPort
               sms.status.value := 'Modem verified successfully'
               sms.modemstatus.fontcolor := { 50, 100, 0 }
               lModemFound := .t.
            endif
         endif
         _sms_conn_close()
      endif
      if .not. lModemFound
         sms.status.value := 'Modem could not be found'
         sms.modemstatus.value := 'Modem Not Verified'
         sms.modemstatus.fontcolor := { 255, 0, 0 }
      endif
   endif
return nil   
         

function _sms_conn_close
   UNINT_PORT( s_nHandle )      
return nil      

function _sms_send_at( cMessage )
   local nTry := 1
   local nMaxTry := sms.tries.value
   local nWait := sms.wait.value * 1000
   local lSuccess := .f.
   local cReceive, nReceive, aLines, cResponse
   if .not. outchr( s_nHandle, cMessage )
      return .f.
   endif
   do while .not. lSuccess .and. nTry <= nMaxTry
      nSize := inbufsize( s_nHandle )
      if nSize > 0
         cReceive := space( nSize )
         nReceive := inchr( s_nHandle, nSize, @cReceive )
         aLines := _comm_StrConvert( cReceive )
         if len( aLines ) > 0
            cResponse := aLines[ len( aLines ) ] 
         endif   
         if cResponse == 'OK'
            lSuccess := .t.
         endif
      endif
      nTry++
      millisec( nWait )
   enddo
   return lSuccess

function _sms_init_msg( cMessage )
   local nTry := 1
   local nMaxTry := sms.tries.value
   local nWait := sms.wait.value * 1000
   local lSuccess := .f.
   local cReceive, nReceive, aLines, cResponse
   if .not. outchr( s_nHandle, cMessage )
      return .f.
   endif
   do while .not. lSuccess .and. nTry <= nMaxTry
      nSize := inbufsize( s_nHandle )
      if nSize > 0
         cReceive := space( nSize )
         nReceive := inchr( s_nHandle, nSize, @cReceive )
         aLines := _comm_StrConvert( cReceive )
         if len( aLines ) > 0
            cResponse := alltrim( aLines[ len( aLines ) ] )
         endif   
         if cResponse == '>'
            lSuccess := .t.
         endif
      endif
      nTry++
      millisec( nWait )
   enddo
   return lSuccess
   
function _sms_send_msg( cMessage )
   local nTry := 1
   local nMaxTry := sms.tries.value
   local nWait := sms.wait.value * 1000
   local lSuccess := .f.
   local cReceive, nReceive, aLines, cResponse
   if .not. outchr( s_nHandle, cMessage )
      return .f.
   endif
   do while .not. lSuccess .and. nTry <= nMaxTry
      nSize := inbufsize( s_nHandle )
      if nSize > 0
         cReceive := space( nSize )
         nReceive := inchr( s_nHandle, nSize, @cReceive )
         cResponse := alltrim( cReceive )
         msginfo( cResponse )
         if at( "+CMGS:", cResponse ) > 0
            lSuccess := .t.
         endif
      endif
      nTry++
      millisec( nWait )
   enddo
   return lSuccess

   
function _comm_StrConvert( cString )
   local aLine := {}
   local tmp
   
   if substr( cString, 1, 2 ) == chr( 13 ) + chr( 10 )
      cString := substr( cString, 3 )
   endif
   IF Right( cString, 2 ) == Chr( 13 ) + Chr( 10 )
      cString := hb_StrShrink( cString, Len( Chr( 13 ) + Chr( 10 ) ) )
   ENDIF

   FOR EACH tmp IN hb_ATokens( StrTran( cString, Chr( 13 ) ), Chr( 10 ) )
      AAdd( aLine, tmp )
   NEXT
   
return aLine


   